﻿Public Class Form1
    ' The column currently used for sorting.
    Private m_SortingColumn As ColumnHeader

    Private Sub Form1_Load() Handles MyBase.Load
        ' Make the ListView column headers.
        ListViewMakeColumnHeaders(lvwBooks,
            "Title", HorizontalAlignment.Left, 120,
            "URL", HorizontalAlignment.Left, 120,
            "ISBN", HorizontalAlignment.Left, 90,
            "Picture", HorizontalAlignment.Left, 120,
            "Pages", HorizontalAlignment.Right, 50,
            "Year", HorizontalAlignment.Right, 40)

        ListViewMakeRow(lvwBooks, 0,
            "Visual Basic and XML",
            "http://www.vb-helper.com/xml.htm",
            "0-471-12060-X",
            "http://www.vb-helper.com/xml.jpg",
            "503",
            "2002")
        ListViewMakeRow(lvwBooks, 0,
            "Visual Basic Graphics Programming, 2e",
            "http://www.vb-helper.com/vbgp.htm",
            "0-471-35599-2",
            "http://www.vb-helper.com/vbgp.jpg",
            "712",
            "2000")
        ListViewMakeRow(lvwBooks, 0,
            "Ready-to-Run Visual Basic Algorithms",
            "http://www.vb-helper.com/vba.htm",
            "0-471-24268-3",
            "http://www.vb-helper.com/vba.jpg",
            "395",
            "1998")

        ListViewSizeColumns(lvwBooks, True)
        Text = "RunTimeListView (" & lvwBooks.View.ToString & ")"
    End Sub

    ' Make the ListView's column headers.
    ' The ParamArray entries should be triples holding
    ' column title, HorizontalAlignment value, and width.
    Private Sub ListViewMakeColumnHeaders(ByVal lvw As ListView, ByVal ParamArray header_info() As Object)
        ' Remove any existing headers.
        lvw.Columns.Clear()

        ' Make the column headers.
        For i As Integer = header_info.GetLowerBound(0) To header_info.GetUpperBound(0) Step 3
            Dim col_header As ColumnHeader = lvw.Columns.Add(
                DirectCast(header_info(i), String),
                -1,
                DirectCast(header_info(i + 1), HorizontalAlignment))
            col_header.Width = DirectCast(header_info(i + 2), Integer)
        Next i
    End Sub

    ' Make a ListView row.
    Private Sub ListViewMakeRow(ByVal lvw As ListView, ByVal image_index As Integer, ByVal item_title As String, ByVal ParamArray subitem_titles() As String)
        ' Make the item.
        Dim new_item As ListViewItem = lvw.Items.Add(item_title)
        new_item.ImageIndex = image_index

        ' Make the sub-items.
        For i As Integer = subitem_titles.GetLowerBound(0) To subitem_titles.GetUpperBound(0)
            new_item.SubItems.Add(subitem_titles(i))
        Next i
    End Sub

    ' Set column widths to -1 to fit data,
    ' -2 to fit data and header.
    Private Sub ListViewSizeColumns(ByVal lvw As ListView, ByVal allow_room_for_header As Boolean)
        Dim new_wid As Integer = -1
        If allow_room_for_header Then new_wid = -2

        ' Set the width for each column.
        For i As Integer = 0 To lvw.Columns.Count - 1
            lvw.Columns(i).Width = new_wid
        Next i
    End Sub

    Private Sub mnuViewDetails_Click() Handles mnuViewDetails.Click
        lvwBooks.View = View.Details
        Text = "RunTimeListView (" & lvwBooks.View.ToString & ")"
    End Sub

    Private Sub mnuViewLargeIcons_Click() Handles mnuViewLargeIcons.Click
        lvwBooks.View = View.LargeIcon
        Text = "RunTimeListView (" & lvwBooks.View.ToString & ")"
    End Sub

    Private Sub mnuViewList_Click() Handles mnuViewList.Click
        lvwBooks.View = View.List
        Text = "RunTimeListView (" & lvwBooks.View.ToString & ")"
    End Sub

    Private Sub mnuViewSmallIcons_Click() Handles mnuViewSmallIcons.Click
        lvwBooks.View = View.SmallIcon
        Text = "RunTimeListView (" & lvwBooks.View.ToString & ")"
    End Sub

    Private Sub mnuViewTile_Click() Handles mnuViewTile.Click
        lvwBooks.View = View.Tile
        Text = "RunTimeListView (" & lvwBooks.View.ToString & ")"
    End Sub

    ' Sort using the clicked column.
    Private Sub lvwBooks_ColumnClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ColumnClickEventArgs) Handles lvwBooks.ColumnClick
        ' Get the new sorting column.
        Dim new_sorting_column As ColumnHeader =
            lvwBooks.Columns(e.Column)

        ' Figure out the new sorting order.
        Dim sort_order As System.Windows.Forms.SortOrder
        If m_SortingColumn Is Nothing Then
            ' New column. Sort ascending.
            sort_order = SortOrder.Ascending
        Else
            ' See if this is the same column.
            If new_sorting_column.Equals(m_SortingColumn) Then
                ' Same column. Switch the sort order.
                If m_SortingColumn.Text.StartsWith("> ") Then
                    sort_order = SortOrder.Descending
                Else
                    sort_order = SortOrder.Ascending
                End If
            Else
                ' New column. Sort ascending.
                sort_order = SortOrder.Ascending
            End If

            ' Remove the old sort indicator.
            m_SortingColumn.Text = m_SortingColumn.Text.Substring(2)
        End If

        ' Display the new sort order.
        m_SortingColumn = new_sorting_column
        If sort_order = SortOrder.Ascending Then
            m_SortingColumn.Text = "> " & m_SortingColumn.Text
        Else
            m_SortingColumn.Text = "< " & m_SortingColumn.Text
        End If

        ' Create a comparer.
        lvwBooks.ListViewItemSorter = New ListViewComparer(e.Column, sort_order)

        ' Sort.
        lvwBooks.Sort()
    End Sub

End Class

' Implements a comparer for ListView columns.
Public Class ListViewComparer
    Implements IComparer

    Private m_ColumnNumber As Integer
    Private m_SortOrder As SortOrder

    Public Sub New(ByVal column_number As Integer, ByVal sort_order As SortOrder)
        m_ColumnNumber = column_number
        m_SortOrder = sort_order
    End Sub

    ' Compare the items in the appropriate column
    ' for objects x and y.
    Public Function Compare(ByVal x As Object, ByVal y As Object) _
     As Integer Implements System.Collections.IComparer.Compare
        Dim item_x As ListViewItem = DirectCast(x, ListViewItem)
        Dim item_y As ListViewItem = DirectCast(y, ListViewItem)

        ' Get the sub-item values.
        Dim string_x As String
        If item_x.SubItems.Count <= m_ColumnNumber Then
            string_x = ""
        Else
            string_x = item_x.SubItems(m_ColumnNumber).Text
        End If

        Dim string_y As String
        If item_y.SubItems.Count <= m_ColumnNumber Then
            string_y = ""
        Else
            string_y = item_y.SubItems(m_ColumnNumber).Text
        End If

        ' Compare them.
        If m_SortOrder = SortOrder.Ascending Then
            Return String.Compare(string_x, string_y)
        Else
            Return String.Compare(string_y, string_x)
        End If
    End Function
End Class
