﻿Public Class RowSortingListView
    Inherits ListView

    Public Shadows Event ColumnClick(ByVal column_number As Integer)

    ' Convert the parent class's ColumnClick event
    ' into our version.
    Private Sub RowSortingListView_ColumnClick(ByVal sender As Object, _
     ByVal e As System.Windows.Forms.ColumnClickEventArgs) Handles MyBase.ColumnClick
        RaiseEvent ColumnClick(e.Column)
    End Sub

    ' Shadow the Clear method to remove only Book items.
    Public Shadows Sub Clear()
        For Each item As ListViewItem In Me.Items
            If item.Text = "Book" Then item.Remove()
        Next item
    End Sub

    ' Reimplement the Sorting property.
    Private m_Sorting As SortOrder
    Public Shadows Property Sorting() As SortOrder
        Get
            Return m_Sorting
        End Get
        Set(ByVal Value As SortOrder)
            ' Save the new value.
            m_Sorting = Value

            ' Make a new ListViewItemSorter if necessary.
            If m_Sorting = SortOrder.None Then
                MyBase.ListViewItemSorter = Nothing
            Else
                MyBase.ListViewItemSorter = _
                    New ListViewComparerAllColumns(m_Sorting)
            End If
        End Set
    End Property

#Region "ListViewComparerAllColumns"
    ' Implements a ListViewItem comparer 
    ' that sorts on all columns.
    Private Class ListViewComparerAllColumns
        Implements IComparer

        ' Ascending or Descending.
        Private m_SortOrder As SortOrder

        ' Initialize with a sort order.
        Public Sub New(ByVal sort_order As SortOrder)
            m_SortOrder = sort_order
        End Sub

        ' Compare two items' subitems.
        Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer _
         Implements System.Collections.IComparer.Compare
            ' Get the ListViewItems.
            Dim item_x As ListViewItem = DirectCast(x, ListViewItem)
            Dim item_y As ListViewItem = DirectCast(y, ListViewItem)

            ' Get the ListViewItems' values.
            Dim values_x As String = ListViewItemValue(item_x)
            Dim values_y As String = ListViewItemValue(item_y)

            ' Compare the values.
            If m_SortOrder = SortOrder.Ascending Then
                Return String.Compare(values_x, values_y)
            Else
                Return String.Compare(values_y, values_x)
            End If
        End Function

        ' Return a delimited string containing all of
        ' the ListViewItem's values.
        Private Function ListViewItemValue(ByVal lvi As ListViewItem, _
         Optional ByVal delimiter As String = vbTab) As String
            Dim txt As String = ""
            For i As Integer = 0 To lvi.SubItems.Count - 1
                txt &= delimiter & lvi.SubItems(i).Text
            Next i
            Return txt.Substring(delimiter.Length)
        End Function
    End Class
#End Region ' ListViewComparerAllColumns

End Class
